package org.jboss.qa.brms.fusionunlinking;

import java.util.concurrent.TimeUnit;

import org.drools.KnowledgeBase;
import org.drools.KnowledgeBaseConfiguration;
import org.drools.KnowledgeBaseFactory;
import org.drools.builder.KnowledgeBuilder;
import org.drools.builder.KnowledgeBuilderFactory;
import org.drools.builder.ResourceType;
import org.drools.builder.conf.LRUnlinkingOption;
import org.drools.conf.EventProcessingOption;
import org.drools.io.ResourceFactory;
import org.drools.runtime.KnowledgeSessionConfiguration;
import org.drools.runtime.StatefulKnowledgeSession;
import org.drools.runtime.conf.ClockTypeOption;
import org.drools.runtime.rule.WorkingMemoryEntryPoint;
import org.drools.time.SessionPseudoClock;

public class SimpleFusionTest {

    public static void main(String[] args) {
        new SimpleFusionTest().testAfterOperator();

        System.out.println("Finished...");
    }

    public void testAfterOperator() {
        KnowledgeSessionConfiguration ksconf = KnowledgeBaseFactory.newKnowledgeSessionConfiguration();
        ksconf.setOption(ClockTypeOption.get("pseudo"));
        StatefulKnowledgeSession ksession = createKnowledgeBase().newStatefulKnowledgeSession(ksconf, null);

        WorkingMemoryEntryPoint stream = ksession.getWorkingMemoryEntryPoint("Transactions Stream");
        SessionPseudoClock clock = ksession.getSessionClock();

        for(int i = 0; i < 3; i++) {
            TransactionCreated tc = new TransactionCreated(i);
            stream.insert(tc);
            clock.advanceTime(8, TimeUnit.SECONDS);
        }
        ksession.fireAllRules();
    }

    protected KnowledgeBase createKnowledgeBase() {
        KnowledgeBuilder kbuilder = KnowledgeBuilderFactory.newKnowledgeBuilder();
        kbuilder.add(ResourceFactory.newClassPathResource("after-before-rule.drl", getClass()), ResourceType.DRL);

        if (kbuilder.hasErrors()) {
            throw new RuntimeException(kbuilder.getErrors().toString());
        }
        KnowledgeBaseConfiguration kbconf = KnowledgeBaseFactory.newKnowledgeBaseConfiguration();
        kbconf.setOption(EventProcessingOption.STREAM);
        kbconf.setOption(LRUnlinkingOption.ENABLED);

        KnowledgeBase kbase = KnowledgeBaseFactory.newKnowledgeBase(kbconf);
        kbase.addKnowledgePackages(kbuilder.getKnowledgePackages());
        return kbase;
    }
}
